/* This program is used to convert a binary ROM dump from an HP 1818-2270 for use in my 1818-2270
ROM adapter. It is needed because the address and data lines on the adapter are all 'flipped' i.e.
A0 is connected to A8, A1 is connected to A7, etc. and similarly D0 is connected to D7, D1 is
connected t0 D6 etc. This program will convert a binary ROM image from an 1818-2270 into one
suitable for programming onto the 27C256 EPROM on the adapter and conversely will convert a ROM
image from the adapter back into a raw 1818-2270 ROM image.

Usage: ./descrambler input.bin output.bin

NOTE: THIS PROGRAM WILL NOT WORK CORRECTLY IF COMPILED ON A BIG-ENDIAN PROCESSOR SUCH AS THE PPC.
*/

#define FILELENGTH 512
#include <stdio.h>

int bitflipper(int input, int numbits){
	int flipped=0;
	int bit=0;
	int i=0;
	for (i=0; i<numbits; i++){
		bit = input & 1;
		input = input >> 1;
		flipped = flipped ^ bit;
		if (i<numbits-1){
			flipped = flipped << 1;
		}
	}
	return flipped;
}

int main( int argc, char *argv[] ){
	if (argc != 3){
        	printf( "usage: %s input filename output filename\n", argv[0] );
	}
	else{
		FILE *file = fopen( argv[1], "r" );
	if ( file == 0 ){
            	printf( "Could not open file\n" );
	}
	else{
		int filecontents[FILELENGTH];
		int outputfile[FILELENGTH];
		int i=0;
		int readchars=0;
		while  ( ( filecontents[readchars] = fgetc( file ) ) != EOF && (readchars<FILELENGTH)){
			readchars++;
		}
		if (readchars != FILELENGTH){
			printf("File is not the right size (512 bytes) to be a binary 1818-2270 ROM image\n");
			return;
		}
		fclose( file );
		for(i=0; i< readchars; i++){
			outputfile[bitflipper(i,9)]=bitflipper(filecontents[i],8);
		}
		FILE *file = fopen(argv[2], "w" );
		for(i=0; i< readchars; i++){
			fputc(outputfile[i], file);
		}
	     	fclose( file );
        }
    }

}



